@@ -44,6 +44,7 @@ from liquid.token import TOKEN_FLOAT
 from liquid.token import TOKEN_IDENTIFIER
 from liquid.token import TOKEN_LPAREN
 from liquid.token import TOKEN_COLON
+from liquid.token import TOKEN_RANGE_LITERAL
 
 
 TOKEN_MAP = {
@@ -75,7 +76,18 @@ def parse_obj(stream: TokenStream) -> Expression:
         ) from err
 
 
-TOKEN_MAP[TOKEN_LPAREN] = make_parse_range(parse_obj)
+parse_range = make_parse_range(parse_obj)
+
+
+def parse_range_with_parens(stream: TokenStream) -> Expression:
+    """Like `parse_range` but consumes the extra `RANGE_LPAREN` token first."""
+    stream.expect(TOKEN_RANGE_LITERAL)
+    next(stream)  # Eat extra token
+    return parse_range(stream)
+
+
+TOKEN_MAP[TOKEN_LPAREN] = parse_range
+TOKEN_MAP[TOKEN_RANGE_LITERAL] = parse_range_with_parens
 
 
 def split_at_pipe(tokens: Iterable[Token]) -> Iterator[List[Token]]:
@@ -105,7 +117,7 @@ def split_at_first_pipe(tokens: Iterable[Token]) -> Iterator[List[Token]]:
 def split_at_comma(
     tokens: Iterable[Token],
 ) -> Iterator[List[Token]]:  # pragma: no cover
-    """Split tokens on into lists, using TOKEN_COMMA as the delimiter."""
+    """Split tokens into lists, using TOKEN_COMMA as the delimiter."""
     buf: List[Token] = []
     for token in tokens:
         if token[1] == TOKEN_COMMA:
@@ -213,10 +225,9 @@ def _parse_filter(tokens: List[Token], linenum: int) -> Filter:
     return Filter(name, args, kwargs)
 
 
-def parse(expr: str, linenum: int = 1) -> FilteredExpression:
-    """Parse an expression string with zero or more filters."""
-    tokens = tokenize(expr, linenum)
-    parts = list(split_at_first_pipe(tokens))
+def parse_from_tokens(tokens: Iterator[Token], linenum: int = 1) -> FilteredExpression:
+    """Parse an expression with zero or more filters from a token iterator."""
+    parts = tuple(split_at_first_pipe(tokens))
     if len(parts) == 1:
         stream = TokenStream(iter(parts[0]))
         return FilteredExpression(parse_obj(stream))
@@ -224,3 +235,8 @@ def parse(expr: str, linenum: int = 1) -> FilteredExpression:
     left = parse_obj(TokenStream(iter(parts[0])))
     filters = [_parse_filter(_tokens, linenum) for _tokens in split_at_pipe(parts[1])]
     return FilteredExpression(left, filters)
+
+
+def parse(expr: str, linenum: int = 1) -> FilteredExpression:
+    """Parse an expression string with zero or more filters."""
+    return parse_from_tokens(tokenize(expr, linenum))
